/***
* MVA Tutorials - www.yookos.com/community/mva for more...
***/
// First we set up the database
var MyPrayerDiaryDB; //global variable for our database

function initDatabase() {
	try {
	    if (!window.openDatabase) {
	        alert('Databases are not supported in this browser.');
	    } else {
	        var shortName = 'MyPrayerDiaryDB';
	        var version = '1.0';
	        var displayName = 'My Prayer Diary Database';
	        var maxSize = 100000; //  bytes
	        // next you use the openDatabase() method to open (or create if it doesn't exist) a db as below:
	        MyPrayerDiaryDB = openDatabase(shortName, version, displayName, maxSize); // This is very important
	        // next we include a function to create a table to hold records of our prayer entries
			createPrayerTable();
			// ...and also read from the table in the database
			readAllPrayers();
	    }
	} catch(e) {
 
	    if (e == 2) {
	        // Version number mismatch.
	        console.log("Invalid database version.");
	    } else {
	        console.log("Unknown error "+e+".");
	    }
	    return;
	}
}

// Then we write our createTable() function. This sets up a table to receive data about the prayer entries
// i.e : prayer, type or prayer, due date, answered ... Feel free to add your own fields.

function createPrayerTable(){
	MyPrayerDiaryDB.transaction(
        function (transaction) {
        	//then we write the SQL instruction - pretty easy... The one below creates a table called prayers if it
        	//doesn't already exists. With fields: id (an Integer, and primary key), prayer (TEXT datatype without null values)...
        	//we store the SQL string in the CREATE_TABLE_SQL variable for a neater work
        	var CREATE_TABLE_SQL = "CREATE TABLE IF NOT EXISTS PRAYERS_TABLE(prayer_id INTEGER PRIMARY KEY AUTOINCREMENT, prayer TEXT NOT NULL, prayer_type TEXT, date_created TEXT, due_date TEXT, answered TEXT)";
        	//then we execute the SQL  using the 'executeSql()' method like so:
        	transaction.executeSql(CREATE_TABLE_SQL, [], nullDataHandler, errorHandler);
        	//the nullDataHandler and errorHandler functions handle null results and error results respectively.
        }
    );
}



function nullDataHandler(){ /* do anything */} // Function for handling null result
function errorHandler(transaction, e) // Function for handling error...
{   alert(e.message);}


// Next, we write a function to display all prayer entries
function readAllPrayers(){
	MyPrayerDiaryDB.transaction(
		function(transaction){
			//Again we store the SQL string in a READ_ALL_PRAYERS variable.
			//Again, it's a pretty simple instruction - we select all entries from the PRAYERS_TABLE we created earlier and sort the results by the prayer_id in descending order
			var READ_ALL_PRAYERS = "SELECT * FROM PRAYERS_TABLE ORDER BY prayer_id DESC"
			// then we execute
			transaction.executeSql(READ_ALL_PRAYERS, [], resultHandler);
			//the resultHandler function will handle the results received
		});

}

//It gets interesting from here. Please pay close attention.
//Now we have to decide what to do with the results from our readAllPrayers function.
//We can append each row of data to the list (<ul id="prayer_list">) in our index.html
//Okay, let's work

function resultHandler(transaction, result){
	//the 'result.rows' from the 'result' parameter is an object containing an array of all prayer entries
	var dataset = result.rows; //we store the array (result.rows) in a variable 'dataset'
    //let's prepare variables to hold different fields in our row.
    var prayer_id = null; prayer = null; prayer_type = null; date_created = null; due_date = null; answered = null;
    
    //Let's do a little check - if there are no prayer entries yet, let's inform the user. This makes our app more intuitive.
    //We will do this with a little if statement
    if (dataset.length == 0) {
    	$("#empty").text("Hello. You currently have an empty diary. Tap \'Add Prayer\' to begin") // we put this message in the <h4 id="empty"> element
    }
    else{
    	$("#empty").text("");
    }

    //then we empty the <ul id="prayer_list"> as a precaution
    $("#prayer_list").empty();

    // next we loop through each row. Variable 'item' represents one single row
	for (var i = 0, item = null; i < dataset.length; i++) {
	    item = dataset.item(i); //a single row

	    prayer_id = item['prayer_id']; // prayer_id field in a row
	    prayer = item['prayer']; // prayer field in a row
	    prayer_type = item['prayer_type']; // prayer_type field in a row
	    date_created = item['date_created']; // date_created field in a row
	    due_date = item['due_date']; // due_date field in a row
	    answered = item['answered']; // answered field in a row

    	// next we append an HTML list item (<li>) containing the above info to the <ul id="prayer_list">
	    
	    var prayerRow = '<li class="ui-btn">'+
	    				'<h4>' + prayer +'</h4>'+
    					'<p>Prayer of: ' + prayer_type +'<br>'+
    					'Date created: ' + date_created + '<br>'+
    					'Expected: ' + due_date + '<br>'+
    					'Status: ' + answered + '</p>'+
    					'<a data-prayer_id=\"'+prayer_id+'\" class="ui-btn ">Update'+ // we'll need the value of the data-prayer_id attribute we just created later for updating
	    				'</a>'+
    					'</li>' ;

	    $("#prayer_list").append(prayerRow); //appends the list item to the index.html prayer_list
    } 

}

//All right here's a function to insert a row of prayer data
function addPrayers(){
	MyPrayerDiaryDB.transaction(function(transaction){
		// First, we'll store all the values of the form <form id="add_prayer_form"> in variables
		var my_prayer = $('#my_prayer').val();
		var my_prayer_type = $('#my_prayer_type').val();
		var my_due_date = $('#my_due_date').val();
		var d = new Date(); // gives us the current date this record was created.. i.e now
		var created_date = d.toDateString(); 
		var answered = "In progress" // initial status of prayer

		//so we store the SQL string in a ADD_PRAYERS variable
		//Again, it's a pretty simple instruction - we insert form data to the the PRAYERS_TABLE we created earlier

		var ADD_PRAYERS = "INSERT INTO PRAYERS_TABLE (prayer, prayer_type, date_created, due_date, answered) VALUES (?, ?, ?, ?, ?)";

		//then we execute while putting in the form values
		transaction.executeSql(ADD_PRAYERS, [my_prayer,my_prayer_type,created_date,my_due_date,answered], successDataHandler, errorHandler);
		// the successHandler will handle successful entries
	});
}

function successDataHandler(){
	alert("Yay! Prayer entry successfully added!");
}

// Almost done with the requirements of our app. Now, we need users to be able to update their prayer entries
// Specifically to update the 'answered' status of a prayer entry
function updatePrayer(prayer_id){ // the prayer_id parameter will be gotten from the data-prayer_id attribute we added earlier
	MyPrayerDiaryDB.transaction(
		function(transaction){
			//As usual we store the SQL string in a variable. We'll call it UPDATE_PRAYER this time
			//Again, it's a pretty simple instruction - we update an entry from the PRAYERS_TABLE
			var UPDATE_PRAYER = "UPDATE PRAYERS_TABLE SET answered=? WHERE prayer_id ="+prayer_id
			// then we execute
			transaction.executeSql(UPDATE_PRAYER, ["Answered"], updateHandler);
			//the updateHandler function will handle the results received

			//then we refresh the list with the readAllPrayers function we created earlier
			readAllPrayers();
		});

}

function updateHandler(){
	alert("Yay! successfully updated prayer!");
}


//Now let's tie up all the features we've coded above to certain events in the app

//First, when the document is created, we'll initialize our database like so:
$(document).on('ready', function(event) {
	initDatabase();
});

//Second, when the index.html file page: <div data-role="page" id="home_page"> shows up.
//this takes JQuery's ajax into consideration
$(document).on('pageshow', '#home_page', function(event) {
	initDatabase();
});

//Third, when a list item button is tapped - for update
$(document).on('click', '#prayer_list li a', function(event) {
	var list = $(this);
	var answered_status = confirm("Update prayer status to answered?"); //an alert box with 2 options
	if (answered_status == true) {
		// if true, then run updatePrayer function
		var prayer_id = list.data('prayer_id'); //the data-id attribute of the list item (remember we appended the prayer to it in the readAllPrayers function)
		updatePrayer(prayer_id);
	};
});



//Finally, when the submit button is clicked in the add_prayer.html page
$(document).on('submit', '#add_prayer_form', function(event) {
	event.preventDefault();
	addPrayers();
	readAllPrayers();//refresh list
	return false;
});



//That's all fellows! We've got our app ready!

//here's an extra for delete operations
function extraDelete(){
	var DELETE = "DROP TABLE PRAYERS_TABLE";
	MyPrayerDiaryDB.transaction(function(transaction){
		transaction.executeSql(DELETE,null,null);
	});
}








